@@ -15,7 +15,7 @@ except ImportError:
 from chia.rpc.rpc_server import start_rpc_server
 from chia.server.outbound_message import NodeType
 from chia.server.server import ChiaServer
-from chia.server.upnp import upnp_remap_port
+from chia.server.upnp import UPnP
 from chia.types.peer_info import PeerInfo
 from chia.util.chia_logging import initialize_logging
 from chia.util.config import load_config, load_config_cli
@@ -108,6 +108,7 @@ class Service:
         self._on_connect_callback = on_connect_callback
         self._advertised_port = advertised_port
         self._reconnect_tasks: List[asyncio.Task] = []
+        self.upnp: Optional[UPnP] = None
 
     async def start(self, **kwargs) -> None:
         # we include `kwargs` as a hack for the wallet, which for some
@@ -115,7 +116,7 @@ class Service:
         # and should be fixed at some point.
         # TODO: move those parameters to `__init__`
         if self._did_start:
-            return
+            return None
 
         assert self.self_hostname is not None
         assert self.daemon_port is not None
@@ -127,7 +128,10 @@ class Service:
         await self._node._start(**kwargs)
 
         for port in self._upnp_ports:
-            upnp_remap_port(port)
+            if self.upnp is None:
+                self.upnp = UPnP()
+
+            self.upnp.remap(port)
 
         await self._server.start_server(self._on_connect_callback)
 
@@ -189,6 +193,10 @@ class Service:
 
                 self._rpc_close_task = asyncio.create_task(close_rpc_server())
 
+            for port in self._upnp_ports:
+                if self.upnp is not None:
+                    self.upnp.release(port)
+
     async def wait_closed(self) -> None:
         await self._is_stopping.wait()
 
