@@ -72,7 +72,7 @@ def debug_spend_bundle(spend_bundle: SpendBundle) -> None:
                 pks.append(pk)
                 msgs.append(m)
             print()
-            r = puzzle_reveal.run(solution)
+            r = puzzle_reveal.run_with_cost(INFINITE_COST, solution)
             print(disassemble(r))
             print()
             if conditions and len(conditions) > 0:
