@@ -24,6 +24,7 @@ from chia.util.streamable import recurse_jsonify
 from chia.wallet.block_record import HeaderBlockRecord
 from chia.wallet.wallet_block_store import WalletBlockStore
 from chia.wallet.wallet_coin_store import WalletCoinStore
+from chia.wallet.wallet_transaction_store import WalletTransactionStore
 
 log = logging.getLogger(__name__)
 
@@ -58,6 +59,7 @@ class WalletBlockchain(BlockchainInterface):
     __sub_epoch_summaries: Dict[uint32, SubEpochSummary] = {}
     # Unspent Store
     coin_store: WalletCoinStore
+    tx_store: WalletTransactionStore
     # Store
     block_store: WalletBlockStore
     # Used to verify blocks in parallel
@@ -77,6 +79,8 @@ class WalletBlockchain(BlockchainInterface):
     @staticmethod
     async def create(
         block_store: WalletBlockStore,
+        coin_store: WalletCoinStore,
+        tx_store: WalletTransactionStore,
         consensus_constants: ConsensusConstants,
         coins_of_interest_received: Callable,  # f(removals: List[Coin], additions: List[Coin], height: uint32)
         reorg_rollback: Callable,
@@ -88,7 +92,9 @@ class WalletBlockchain(BlockchainInterface):
         in the consensus constants config.
         """
         self = WalletBlockchain()
-        self.lock = asyncio.Lock()  # External lock handled by full node
+        self.lock = asyncio.Lock()
+        self.coin_store = coin_store
+        self.tx_store = tx_store
         cpu_count = multiprocessing.cpu_count()
         if cpu_count > 61:
             cpu_count = 61  # Windows Server 2016 has an issue https://bugs.python.org/issue26903
@@ -126,7 +132,7 @@ class WalletBlockchain(BlockchainInterface):
         if len(blocks) == 0:
             assert peak is None
             self._peak_height = None
-            return
+            return None
 
         assert peak is not None
         self._peak_height = self.block_record(peak).height
@@ -228,7 +234,10 @@ class WalletBlockchain(BlockchainInterface):
                     await self.block_store.db_wrapper.commit_transaction()
                 except BaseException as e:
                     self.log.error(f"Error during db transaction: {e}")
-                    await self.block_store.db_wrapper.rollback_transaction()
+                    if self.block_store.db_wrapper.db._connection is not None:
+                        await self.block_store.db_wrapper.rollback_transaction()
+                        await self.coin_store.rebuild_wallet_cache()
+                        await self.tx_store.rebuild_tx_cache()
                     raise
             if fork_height is not None:
                 self.log.info(f"💰 Updated wallet peak to height {block_record.height}, weight {block_record.weight}, ")
@@ -271,7 +280,8 @@ class WalletBlockchain(BlockchainInterface):
 
             # Rollback to fork
             self.log.debug(f"fork_h: {fork_h}, SB: {block_record.height}, peak: {peak.height}")
-            await self.reorg_rollback(fork_h)
+            if block_record.prev_hash != peak.header_hash:
+                await self.reorg_rollback(fork_h)
 
             # Rollback sub_epoch_summaries
             heights_to_delete = []
@@ -377,7 +387,7 @@ class WalletBlockchain(BlockchainInterface):
         """
 
         if self._peak_height is None:
-            return
+            return None
         blocks = await self.block_store.get_block_records_in_range(
             fork_point - self.constants.BLOCKS_CACHE_SIZE, self._peak_height
         )
@@ -392,7 +402,7 @@ class WalletBlockchain(BlockchainInterface):
         """
 
         if height < 0:
-            return
+            return None
         blocks_to_remove = self.__heights_in_cache.get(uint32(height), None)
         while blocks_to_remove is not None and height >= 0:
             for header_hash in blocks_to_remove:
@@ -410,18 +420,20 @@ class WalletBlockchain(BlockchainInterface):
         """
 
         if len(self.__block_records) < self.constants.BLOCKS_CACHE_SIZE:
-            return
+            return None
 
         peak = self.get_peak()
         assert peak is not None
         if peak.height - self.constants.BLOCKS_CACHE_SIZE < 0:
-            return
+            return None
         self.clean_block_record(peak.height - self.constants.BLOCKS_CACHE_SIZE)
 
     async def get_block_records_in_range(self, start: int, stop: int) -> Dict[bytes32, BlockRecord]:
         return await self.block_store.get_block_records_in_range(start, stop)
 
-    async def get_header_blocks_in_range(self, start: int, stop: int) -> Dict[bytes32, HeaderBlock]:
+    async def get_header_blocks_in_range(
+        self, start: int, stop: int, tx_filter: bool = True
+    ) -> Dict[bytes32, HeaderBlock]:
         return await self.block_store.get_header_blocks_in_range(start, stop)
 
     async def get_block_record_from_db(self, header_hash: bytes32) -> Optional[BlockRecord]:
