@@ -7,6 +7,11 @@ if [ -z "$VIRTUAL_ENV" ]; then
 	exit 1
 fi
 
+if [ "$(id -u)" = 0 ]; then
+  echo "The Chia Blockchain GUI can not be installed or run by the root user."
+	exit 1
+fi
+
 # Allows overriding the branch or commit to build in chia-blockchain-gui
 SUBMODULE_BRANCH=$1
 
@@ -18,17 +23,23 @@ if [ "$(uname)" = "Linux" ]; then
 		# Debian/Ubuntu
 		UBUNTU=true
 		sudo apt-get install -y npm nodejs libxss1
-	elif type yum && [ ! -f "/etc/redhat-release" ] && [ ! -f "/etc/centos-release" ]; then
+	elif type yum &&  [ ! -f "/etc/redhat-release" ] && [ ! -f "/etc/centos-release" ] && [ ! -f /etc/rocky-release ]; then
 		# AMZN 2
-		echo "Installing on Amazon Linux 2"
+		echo "Installing on Amazon Linux 2."
 		curl -sL https://rpm.nodesource.com/setup_12.x | sudo bash -
 		sudo yum install -y nodejs
-	elif type yum && [ -f /etc/redhat-release ] || [ -f /etc/centos-release ]; then
+	elif type yum && [ ! -f /etc/rocky-release ] && [ -f /etc/redhat-release ] || [ -f /etc/centos-release ]; then
 		# CentOS or Redhat
-		echo "Installing on CentOS/Redhat"
+		echo "Installing on CentOS/Redhat."
 		curl -sL https://rpm.nodesource.com/setup_12.x | sudo bash -
 		sudo yum install -y nodejs
-	fi
+	elif type yum && [ -f /etc/rocky-release ]; then
+                # RockyLinux
+                echo "Installing on RockyLinux"
+                dnf module enable nodejs:12
+                sudo dnf install -y nodejs
+        fi
+
 elif [ "$(uname)" = "Darwin" ] && type brew && ! npm version >/dev/null 2>&1; then
 	# Install npm if not installed
 	brew install npm
@@ -46,7 +57,7 @@ if $UBUNTU; then
 fi
 
 if [ "$UBUNTU_PRE_2004" = "True" ]; then
-	echo "Installing on Ubuntu older than 20.04 LTS: Ugrading node.js to stable"
+	echo "Installing on Ubuntu older than 20.04 LTS: Ugrading node.js to stable."
 	UBUNTU_PRE_2004=true # Unfortunately Python returns True when shell expects true
 	sudo npm install -g n
 	sudo n stable
@@ -54,12 +65,12 @@ if [ "$UBUNTU_PRE_2004" = "True" ]; then
 fi
 
 if [ "$UBUNTU" = "true" ] && [ "$UBUNTU_PRE_2004" = "False" ]; then
-	echo "Installing on Ubuntu 20.04 LTS or newer: Using installed node.js version"
+	echo "Installing on Ubuntu 20.04 LTS or newer: Using installed node.js version."
 fi
 
-# We will set up node.js on GitHub Actions and Azure Pipelines directly
-# for Mac and Windows so skip unless completing a source/developer install
-# Ubuntu special cases above
+# For Mac and Windows, we will set up node.js on GitHub Actions and Azure
+# Pipelines directly, so skip unless you are completing a source/developer install.
+# Ubuntu special cases above.
 if [ ! "$CI" ]; then
 	echo "Running git submodule update --init --recursive."
 	echo ""
@@ -83,10 +94,10 @@ if [ ! "$CI" ]; then
 	npm audit fix || true
 	npm run build
 else
-	echo "Skipping node.js in install.sh on MacOS ci"
+	echo "Skipping node.js in install.sh on MacOS ci."
 fi
 
 echo ""
-echo "Chia blockchain install-gui.sh complete."
+echo "Chia blockchain install-gui.sh completed."
 echo ""
-echo "Type 'cd chia-blockchain-gui' and then 'npm run electron &' to start the GUI"
+echo "Type 'cd chia-blockchain-gui' and then 'npm run electron &' to start the GUI."
